"""
Module realizes VLException, exception for internal usages
"""

from functools import wraps
from typing import Optional

from luna3.common.exceptions import LunaApiException
from vlutils.helpers import isCoroutineFunction

from .errors import ErrorInfo


class VLException(Exception):
    """
    VLException

    Attributes:
        error: error
        statusCode: status code for response
        isCriticalError: print trace or not
        headers: additional http headers
    """

    def __init__(
        self, error: ErrorInfo, statusCode=500, isCriticalError=True, exception=None, headers: Optional[dict] = None
    ):
        super().__init__(error.description)
        self.error = error
        self.statusCode = statusCode
        self.isCriticalError = isCriticalError
        self.exception = exception
        self.headers = headers


def exceptionWarp(error):
    """
    Decorator for catching exceptions and raise VLException with specific error.

    Args:
        error: returning error in the exception case
    Returns:
        if exception was caught, system calls error method with error
    Raises:
        VLException(error, exception=e): if catch any exception except VLException, LunaApiException
    """

    def realWarp(func):
        if isCoroutineFunction(func):

            @wraps(func)
            async def wrap(*func_args, **func_kwargs):
                try:
                    res = await func(*func_args, **func_kwargs)
                    return res
                except (VLException, LunaApiException, TypeError):
                    raise
                except Exception as exc:
                    func_args[0].logger.exception()
                    raise VLException(error, exception=exc) from exc

        else:

            @wraps(func)
            def wrap(*func_args, **func_kwargs):
                try:
                    res = func(*func_args, **func_kwargs)
                    return res
                except (VLException, LunaApiException, TypeError):
                    raise
                except Exception as exc:
                    func_args[0].logger.exception()
                    raise VLException(error, exception=exc) from exc

        return wrap

    return realWarp
